//test_invariant.cpp

#define STRICT

#include "..\invariants.hpp"

using namespace boost::DBC;

class my_class1
{
 protected:
    int a, b;
 private:
    INVARIANTS(my_class1)
    {
        assert( a + b == 5 );
    }
 public:
    my_class1() : a(2), b(3)
    {
        INV_CHECK();
    }
    ~my_class1()
    {
        INV_CHECK();
    }
    void f()
    {
        INV_CHECK();
        ++a; --b;
    }
    void g()
    {
        INV_CHECK();
        ++a;
    }
};

class my_class2
{
 protected:
    int c, d;
 private:
    INVARIANTS(my_class2)
    {
        assert( c + d == 9 );
    }
 public:
    my_class2() : c(5), d(4)
    {
        INV_CHECK();
    }
    ~my_class2()
    {
        INV_CHECK();
    }
    void h()
    {
        INV_CHECK();
        ++d;
    }
    void i()
    {
        INV_CHECK();
        --c; ++d;
    }
};

template< typename I, int J >
class my_class3 : public my_class1, public my_class2
{
    I i_;
    INVARIANTS(my_class3)
    {
        assert( a + c == 7 );
        assert( b + d == 7 );
        assert( !i_ );
    }
 public:
    my_class3() : i_(J)
    {
        INV_CHECK();
    }
    ~my_class3()
    {
        INV_CHECK();
    }
    void j()
    {
        INV_CHECK();
        f(); i();
    }
    void k()
    {
        INV_CHECK();
        h(); g();
    }
    void l()
    {
        INV_CHECK();
        ++a;
    }
    void m()
    {
        INV_CHECK();
        ++i_;
    }
};

int main()
{
    my_class1 my_obj1;
    my_obj1.f();
    //my_obj1.g(); //BOOM!
    my_class2 my_obj2;
    my_obj2.h(); //BOOM!
    my_obj2.i();
    my_class3<int,0> my_obj3;
    my_obj3.f();
    my_obj3.g(); //BOOM!
    my_obj3.h(); //BOOM!
    my_obj3.i();
    my_obj3.j();
    my_obj3.k(); //BOOM!
    my_obj3.l(); //BOOM!
    my_obj3.m(); //BOOM!
    return 0;
}


